import { httpApi } from '../../request/httpApi'

export default {

    //获取数据
    async queryListData(form, msg = true) {
        return httpApi({
            url: '/api/blade-business/dap/list',
            method: 'get',
            params: form,
        }, msg)
    },
    //新增
    async addSave(form, msg = true) {
        return httpApi({
            url: '/api/blade-business/dap/add',
            method: 'post',
            data: form,
        }, msg)
    },
    //编辑
    async updateSave(form, msg = true) {
        return httpApi({
            url: '/api/blade-business/dap/update',
            method: 'post',
            data: form,
        }, msg)
    },
    //删除
    async delData(form, msg = true) {
        return httpApi({
            url: '/api/blade-business/dap/del',
            method: 'post',
            params: form,
        }, msg)
    },
    //导出数据
    async getExportExcel(form, msg = true) {
        return httpApi({
            url: '/api/blade-business/dap/exportExcel',
            method: 'get',
            params: form,
            responseType: 'blob',
        }, msg)
    },
}
